//
// C++ Implementation: balkendiagram
//
// Description:
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "balkendiagram.h"
#include <QPainter>
#include <QInputDialog>
#include <QColorDialog>
#include <QtDebug>

BalkenDiagram::BalkenDiagram(QWidget* parent): Diagram(parent)
{
	colors[0]=Qt::blue;
	colors[1]=Qt::red;
}


BalkenDiagram::~BalkenDiagram()
{}


void BalkenDiagram::paintEvent(QPaintEvent *ev)
{
	using namespace std;

	if (data.size()==0) return; // Keine Daten, also nicht zeichnen

	QPainter p(this);

	p.fillRect(rect(),Qt::white);

	QFont f1 = p.font();
	QFont f2 = p.font();

	// 1. Maximum finden
	double max = *max_element(data.begin(),data.end()); // Kurzfassung
	
	// Titel
	QFont hf(p.font()),of(p.font());
	hf.setBold(true);
	hf.setPointSize(hf.pointSize()+1);
	
	QFontMetrics fm(hf);
	p.setFont(hf);
	if (fm.width(_title)>(width()-10))
		p.drawText(5,fm.height(),_title);
	else
		p.drawText((width()-fm.width(_title))/2,fm.height(),_title);
	p.setFont(of);
	
	//2. vmax berechnen
	int vmax = height()-25 -fm.height() - p.fontMetrics().height();
	
	// Zeilen
	int rows,cols;
	if (row_width>0)
	{
		rows = data.size()/row_width;
		cols = row_width;
	}
	else
	{
		rows=1;
		cols = data.size();
	}
	
	int dwidth=width(); // Nutzbare Breite
	int dmin=5;
	
	// Legende
	if (colLabels.count()>0)
	{
		int ms = 0;
		for (int i=0;i<colLabels.count();++i) ms = qMax(ms,p.fontMetrics().width(colLabels[i])); // Maximale Breite bestimmen
		ms += 10; // Rand
		dwidth -= ms+5;
		int h = p.fontMetrics().height()*(colLabels.count())+5;
		QRect rr(width()-ms-5,25,ms,h);
		p.fillRect(rr,Qt::white);
		p.drawRect(rr);
		for (int i=0;i<colLabels.count();++i)
		{
			p.setPen(colors[i%colors.count()]);
			p.drawText(rr.x()+6,rr.y()+(1+i)*p.fontMetrics().height(),colLabels[i]);
		}
	}
	
	// Linien
	p.setPen(Qt::black);
	if(value_postfix!="%")
		for (int y = height()-20; y>height()-vmax-20;y-=(height()-50)/5) p.drawLine(5,y,dwidth-5,y);
	else
	{
		// Prozent modus
		int step=2,count=5;
		if(max>75) {step=25;max=100;count=4;}
		else if(max>50) {step=25;max=75;count=3;}
		else if(max>30) {step=10;max=50;count=5;}
		else if(max>20) {step=5;max=30;count=6;}
		else if(max>10) {step=4;max=20;count=5;}
		else {max=10;step=2;count=5;}
		
		for(int i=0;i<=count;++i)
		{
			const int y = height()-20 - (step*i)*vmax/max;
			p.drawLine(5,y,dwidth-5,y);
			p.drawText(5,y,QString("%1%").arg(i*step));
		}
		dmin += p.fontMetrics().width(QString("%1%").arg(max));
	}
	
	dwidth -= dmin;
	
	//4 b berechnen

	int b;
	if(rows>1) b=dwidth/(1.5*(1+rows)*cols-1);
	else b = dwidth/(1.5*cols+0.5);
	
	vmax -= b*0.4; // Platz fuer 3d Effekt lassen
	
	//6. x als Startwert definieren

	int x=b/2+dmin;

	int i=0,r=0; // Laufende Nummer

	QColor c = colors[0];
	
	// Breite der Werte Testen
	bool drawValues=true;
	for(int i=0;i<data.size();++i)
	{
		const QString t = QString("%1%2").arg(data[i],0,'f',1).arg(value_postfix);//QString::number(data[i])+value_postfix;
		const int tw = p.fontMetrics().width(t); // Textbreite
		if(tw>=b) drawValues=false;
	}
	
	// Zeichnen
	
	

	for (QVector<double>::iterator it = data.begin();it!=data.end();++it)
	{
		double v = *it;
		//3. Vx berechnen
		int vx=v/max*vmax;
		//5. Saeule malen
		p.fillRect(x,height()-vx-20,b,vx,c);
		
		QPoint ps[]={QPoint(x+b,height()-20),QPoint(x+b,height()-vx-20),QPoint(x+b*1.353,height()-vx-20-b*0.353),QPoint(x+b*1.353,height()-20-b*0.353)};
		p.setBrush(c.dark());
		p.setPen(c.dark());
		p.drawPolygon(ps,4);
		
		QPoint pt[]={QPoint(x,height()-vx-20),QPoint(x+b,height()-vx-20),QPoint(x+b*1.353,height()-20-vx-b*0.353),QPoint(x+b*0.353,height()-vx-20-b*0.353)};
		p.setBrush(c.light());
		p.setPen(c);
		p.drawPolygon(pt,4);
		
		// Beschriften
		p.setBrush(Qt::white);
		p.setPen(Qt::black);
		p.setFont(f1);
		const QString t = QString("%1%2").arg(v,0,'f',1).arg(value_postfix);//QString::number(v)+value_postfix;
		const int tw = p.fontMetrics().width(t); // Textbreite
		//if (drawValues) p.drawText(x+(b*1.343-tw)/2, fm.height()+p.fontMetrics().height() ,t); // Wertausgeben wenn nicht zu breit
		if (drawValues) p.drawText(x+(b*1.343-tw)/2, height()-vx-20-b*0.4,t); // Wertausgeben wenn nicht zu breit
		if (r==0)
		{
			QString l;
			if (i<labels.count()) l = labels[i]; else l = QString::number(i);
			p.setFont(f2);
			//p.drawText(x+b/4,height()-5,l);  // Spalte
			const int lw = p.fontMetrics().width(l);
			p.drawText(x+((rows)*b-lw)/2,height()-5,l);
		}
		++i;
		// 6. naechster x Wert
		if (rows>1) x+=(1+rows)*b*1.5;
		else x += b*1.5;

		// Neue Zeile
		if (row_width>0 && i>=row_width)
		{
			i=0;
			++r;
			x=b/2+dmin+r*1.5*b;
			c = colors[r%colors.count()];
		}
	}
	p.drawLine(5,height()-20,dwidth-5,height()-20);
}

void BalkenDiagram::colorAction()
{
	int idx=0;
	if(!colLabels.isEmpty())
	{
		bool ok;
		QString l = QInputDialog::getItem(this,tr("Zeile"),tr("Zeile auswhlen deren Farbe sie ndern wollen"),colLabels,0,&ok);
		if(!ok) return;
		idx = colLabels.indexOf(l);
	}
	QColor oc = colors.at(idx%colors.count());
	QColor c = QColorDialog::getColor(oc,this);
	if(c.isValid())	colors[idx%colors.count()]=c;
	update();
}
