//
// C++ Implementation: haeufigkeitsverteilungsadapter
//
// Description: 
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "haeufigkeitsverteilungsadapter.h"
#include "diagram.h"
#include "columnselectdialog.h"

#include <QVector>
#include <numeric>
#include <QStandardItemModel>
#include <QtDebug>
#include <QAction>
#include <QInputDialog>

HaeufigkeitsVerteilungsAdapter::HaeufigkeitsVerteilungsAdapter(QWidget* parent): ModelDiagramAdapter(parent),N(10),input_filter(""),filter_spalte(-1)
{
	QAction *aN = new QAction(tr("Intervale"),this);
	connect(aN,SIGNAL(triggered()),this,SLOT(intervalAction()));
	actions << aN;
	QAction *a = new QAction(tr("mit Spalte verknpfen"),this);
	connect(a,SIGNAL(triggered()),this,SLOT(depAction()));
	actions << a;
	a = new QAction(tr("Verknpfung aufheben"),this);
	connect(a,SIGNAL(triggered()),this,SLOT(undepAction()));
	actions << a;
}


HaeufigkeitsVerteilungsAdapter::~HaeufigkeitsVerteilungsAdapter()
{
}


void HaeufigkeitsVerteilungsAdapter::updateDiagram()
{
	if(!td || !_model) return;
	using namespace std;

	// Werte aus Model einlesen
	QVector<double> werte;
	for(int i=0;i<_model->rowCount();++i)
	{
		if( filter_spalte>=0 && filter_wert!=_model->data(_model->index(i,filter_spalte)).toString()) continue;
		if( !input_filter.isNull() && input_filter == _model->data(_model->index(i,model_column))) continue;
		werte << _model->data(_model->index(i,model_column)).toDouble();
	}
	
	// Grenzen und Interval
	const double min = *min_element(werte.begin(),werte.end());
	const double max = *max_element(werte.begin(),werte.end());
	const double interval = (max-min)/N;
	
	QVector<double> h(N); // Haeufigkeitsverteilung
	QStringList labels;
	int count=0;
	for(double i=min;i<=max;i+=interval,++count)
	{
		labels << QString::number(i+interval/2.0);
		foreach(double v,werte)
		{
			if(i<=v && v<i+interval) h[count]++;
		}
	}
	td->setLabels(labels);
	td->setData(h);
}



/*!
    \fn HaeufigkeitsVerteilungsAdapter::intervalAction()
 */
void HaeufigkeitsVerteilungsAdapter::intervalAction()
{
	bool ok;
	int nN = QInputDialog::getInteger(this,tr("Intervale"),tr("Anzahl der Intervale"),N,6,100,1,&ok);
	if(!ok) return;
	N = nN;
	updateDiagram();
}

void HaeufigkeitsVerteilungsAdapter::setDepFilter(QString Filter_Wert,int Filter_Spalte)
{
	filter_wert = Filter_Wert;
	filter_spalte = Filter_Spalte;
	updateDiagram();
}


/*!
    \fn HaeufigkeitsVerteilungsAdapter::setOutputFilter(const QString& fv)
 */
void HaeufigkeitsVerteilungsAdapter::setOutputFilter(const QVariant& fv)
{
	output_filter = fv;
	updateDiagram();
}


/*!
    \fn HaeufigkeitsVerteilungsAdapter::setInputFilter(const QString& fv)
 */
void HaeufigkeitsVerteilungsAdapter::setInputFilter(const QVariant& fv)
{
	input_filter = fv;
	updateDiagram();
}


/*!
    \fn HaeufigkeitsVerteilungsAdapter::depAction()
 */
void HaeufigkeitsVerteilungsAdapter::depAction()
{
	bool ok;
	int col=-1;
	if(model()) col = ColumnSelectDialog::getColumn(model());
	if(col<0) return;
	QSet<QString> values;
	for(int i=0;i<model()->rowCount();++i) values << model()->data(model()->index(i,col)).toString();
	
	QString v = QInputDialog::getItem(this,tr("Filter"),tr("Wert nach dem gefiltert werden soll"),values.toList(),0,true,&ok);
	
	if(!ok) return;
	setDepFilter(v,col);
	updateDiagram();
}


/*!
    \fn HaeufigkeitsVerteilungsAdapter::undepAction()
 */
void HaeufigkeitsVerteilungsAdapter::undepAction()
{
	setDepFilter(QString(),-1);
}

void HaeufigkeitsVerteilungsAdapter::saveData(QDataStream& s,int ver)
{
	if(ver>1) ModelDiagramAdapter::saveData(s,ver);
	s << input_filter;
	s << filter_wert << (qint32) filter_spalte;
	s << N;
}

void HaeufigkeitsVerteilungsAdapter::loadData(QDataStream& s,int ver)
{
	if(ver>1) ModelDiagramAdapter::loadData(s,ver);
	s >> input_filter;
	QString v;
	int sp;
	s >> v >> sp;
	setDepFilter(v,sp);
	s >> N;
}
