//
// C++ Implementation: test
//
// Description:
//
//
// Author: Arne Schmitt <arne@as4m.de>, (C) 2006
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "modeldiagramadapter.h"
#include "diagram.h"
#include <QLayout>
#include <QStandardItemModel>
#include <QtDebug>
#include <QMenu>

ModelDiagramAdapter::ModelDiagramAdapter(QWidget *parent)
	:ModelClient(parent),td(0),model_column(0)
{
	setMinimumSize(150,120);
	glayout = new QGridLayout(this);
	glayout->setMargin(0);
	glayout->setSpacing(0);
	setContextMenuPolicy(Qt::DefaultContextMenu);
	QAction *a = new QAction(tr("Typ..."),this);
	connect(a,SIGNAL(triggered()),this,SLOT(typAction()));
	actions << a;
}

void ModelDiagramAdapter::setModel(QStandardItemModel *m)
{
	if(_model) disconnect(_model,SIGNAL(dataChanged( const QModelIndex &, const QModelIndex & )),this,SLOT(updateDiagram()));
	_model = m;
	connect(_model,SIGNAL(dataChanged( const QModelIndex &, const QModelIndex &)),this,SLOT(updateDiagram()));
	updateDiagram();
}

void ModelDiagramAdapter::setModelColumn(int c)
{
	model_column = c;
	updateDiagram();
}

void ModelDiagramAdapter::setDiagram(Diagram *d)
{
	if(td) delete td;
	td = d;
	td->setParent(this);
	glayout->addWidget(td,0,0);
	updateDiagram();
}

void ModelDiagramAdapter::setTitle(const QString &t)
{
	_title = t;
	if(td) td->setTitle( t );
}

Diagram *ModelDiagramAdapter::diagram()
{
	return td;
}

QStandardItemModel *ModelDiagramAdapter::model()
{
	return _model;
}

int ModelDiagramAdapter::modelColumn()
{
	return model_column;
}

QVector<QAction*> ModelDiagramAdapter::actionList()
 {
	 return actions;
 }


/*!
    \fn ModelDiagramAdapter::contextMenuEvent ( QContextMenuEvent * event )
 */
void ModelDiagramAdapter::contextMenuEvent ( QContextMenuEvent * event )
{
	QMenu m;
	m.addActions(actions.toList());
	m.addSeparator();
	m.addActions(diagram()->actionList().toList());
	m.exec(event->globalPos());
}

void ModelDiagramAdapter::saveData(QDataStream& s,int ver)
{
	td->saveData(s,ver);
}

void ModelDiagramAdapter::loadData(QDataStream& s,int ver)
{
	td->loadData(s,ver);
}

void ModelDiagramAdapter::typAction()
{
	emit changeDiagram(this);
}
