//
// C++ Implementation: StatistikTabelle
//
// Description:
//
//
// Author: Priyanka Singh <priyanka@india-souring-research.com>, (C) 2007
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "statistiktabelle.h"
#include <numeric>
#include <QStandardItemModel>
#include <QtDebug>
#include <QLayout>
#include <QTableWidget>
#include <QDrag>
#include <QMimeData>
#include <QApplication>
#include "modelclient.h"


StatistikTabelle::StatistikTabelle()
{
	table = new QTableWidget(this);
	QGridLayout *l = new QGridLayout(this);
	l->addWidget(table,0,0);
}


StatistikTabelle::~StatistikTabelle()
{}


void StatistikTabelle::setSpalten(const QVector<int> &s)
{
	Spalten=s;
	updateDiagram();
}

void StatistikTabelle::setModel(QStandardItemModel *m)
{
	if (_model) disconnect(_model,SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)),this,SLOT(updateDiagram()));
	_model = m;
	connect(_model,SIGNAL(dataChanged(const QModelIndex &, const QModelIndex &)),this,SLOT(updateDiagram()));
	updateDiagram();
}

void StatistikTabelle::updateDiagram()
{
	if (!_model || Spalten.size() ==0) return;


	table->setRowCount(7);    // Anzahl der Statistikfunktionen
	table->setColumnCount(Spalten.size());    // Anzahl der Spalten
	for (uint j=0;j<Spalten.size();++j)
	{ // Spaltenueberschriften uebernehmen
		QString h = _model->headerData(Spalten[j],Qt::Horizontal).toString();
		table->setHorizontalHeaderItem(j,new QTableWidgetItem(h));
		//table->setColumnWidth ( j,50 );
	}

	// Zeilenbeschriftung
	table->setVerticalHeaderItem(0,new QTableWidgetItem("Min"));
	table->setVerticalHeaderItem(1,new QTableWidgetItem("Max"));
	table->setVerticalHeaderItem(2,new QTableWidgetItem("Median"));
	table->setVerticalHeaderItem(3,new QTableWidgetItem("Summe"));
	table->setVerticalHeaderItem(4,new QTableWidgetItem("Durchschnitt"));
	table->setVerticalHeaderItem(5,new QTableWidgetItem("Leer %"));
	table->setVerticalHeaderItem(6,new QTableWidgetItem("Werte %"));

	// Spaltenstatistiken berechnen
	for (uint i=0;i<Spalten.size();++i)
	{
		QVector<double> sp;
		for (int j=0;j<_model->rowCount();++j)
		{
			sp << _model->data(_model->index(j,Spalten[i])).toDouble();
		}
		std::sort(sp.begin(),sp.end());
		const double min = *std::min_element(sp.begin(),sp.end());
		const double max = *std::max_element(sp.begin(),sp.end());
		const double median = * (sp.begin() +sp.size() /2);
		const double sum = std::accumulate(sp.begin(),sp.end(),0.0);
		const double avg = sum/sp.size();
		const double em = _model->rowCount()-sp.size();
		const double emp = em/_model->rowCount() *100.0;
		const double vp = sp.count() *100.0/_model->rowCount();

		table->setItem(0, i, new QTableWidgetItem(QString::number(min)));
		table->setItem(1, i, new QTableWidgetItem(QString::number(max)));
		table->setItem(2, i, new QTableWidgetItem(QString::number(median)));
		table->setItem(3, i , new QTableWidgetItem(QString::number(sum)));
		table->setItem(4, i , new QTableWidgetItem(QString::number(avg)));
		table->setItem(5, i , new QTableWidgetItem(QString::number(emp)));
		table->setItem(6, i , new QTableWidgetItem(QString::number(vp)));
	}
}

QString StatistikTabelle::toText()
{
	QString out;
	for(int j=0;j<table->columnCount();++j)
	{
		QTableWidgetItem *it = table->horizontalHeaderItem(j);
		out += "\t";
		out += it->text();
	}
	out += "\n";
	for(int i=0;i<table->rowCount();++i)
	{
		out += table->verticalHeaderItem(i)->text();
		for(int j=0;j<table->columnCount();++j)
		{
			QTableWidgetItem *it = table->item(i,j);
			out += "\t";
			out += it->text();
		}
		out += "\n";
	}
	return out;
}

QString StatistikTabelle::toHTML()
{
	QString out="<TABLE border=1 width=90%>";
	out += "<TR><TD/>";
	for(int j=0;j<table->columnCount();++j)
	{
		QTableWidgetItem *it = table->horizontalHeaderItem(j);
		out += "<TD>"+it->text()+"</TD>";
	}
	out += "</TR>";
	for(int i=0;i<table->rowCount();++i)
	{
		out += "<TR><TD>";
		out += table->verticalHeaderItem(i)->text();
		out += "</TD>";
		for(int j=0;j<table->columnCount();++j)
		{
			QTableWidgetItem *it = table->item(i,j);
			out += "<TD>"+it->text()+"</TD>";
		}
		out += "</TR>";
	}
	out += "</TABLE>";
	return out;
}

// Drag and Drop
 void StatistikTabelle::mousePressEvent(QMouseEvent *event)
 {
     if (event->button() == Qt::LeftButton)
         dragStartPosition = event->pos();
 }


 void StatistikTabelle::mouseMoveEvent(QMouseEvent *event)
 {
     if (!(event->buttons() & Qt::LeftButton))
         return;
     if ((event->pos() - dragStartPosition).manhattanLength()
          < QApplication::startDragDistance())
         return;

     QDrag *drag = new QDrag(this);
     QMimeData *mimeData = new QMimeData;

	TextInhalt *ti = dynamic_cast<TextInhalt*>(this);
	if(ti)
	{
		// Get Text
		mimeData->setHtml(ti->toHTML());
	}
	else
	{
     // Get Image
     QPixmap px = QPixmap::grabWidget(this);
     mimeData->setImageData(px.toImage());
	}
     //mimeData->setData(mimeType, data);
     drag->setMimeData(mimeData);

     Qt::DropAction dropAction = drag->start(Qt::CopyAction | Qt::MoveAction);
 }





